@@ -1,4 +1,4 @@
-"""Parser is a generator function.
+"""Parser is a generator function (NOT coroutine).
 
 Parser receives data with generator's send() method and sends data to
 destination DataQueue. Parser receives ParserBuffer and DataQueue objects
@@ -68,7 +68,7 @@ BUF_LIMIT = 2**14
 DEFAULT_LIMIT = 2**16
 
 
-class StreamParser:
+class StreamParser(asyncio.streams.StreamReader):
     """StreamParser manages incoming bytes stream and protocol parsers.
 
     StreamParser uses ParserBuffer as internal buffer.
@@ -107,7 +107,7 @@ class StreamParser:
         self._stream_paused = True
 
     def resume_stream(self):
-        if self._paused and self._buffer.size <= self._limit:
+        if self._paused and len(self._buffer) <= self._limit:
             self._paused = False
             self._transport.resume_reading()
 
@@ -146,7 +146,7 @@ class StreamParser:
             self._buffer.feed_data(data)
 
         if (self._transport is not None and not self._paused and
-                self._buffer.size > 2*self._limit):
+                len(self._buffer) > 2*self._limit):
             try:
                 self._transport.pause_reading()
             except NotImplementedError:
@@ -174,7 +174,6 @@ class StreamParser:
             self._parser = None
             self._output = None
 
-        self._buffer.shrink()
         self._eof = True
 
     def set_parser(self, parser, output=None):
@@ -212,9 +211,6 @@ class StreamParser:
 
     def unset_parser(self):
         """unset parser, send eof to the parser and then remove it."""
-        if self._buffer:
-            self._buffer.shrink()
-
         if self._parser is None:
             return
 
@@ -285,8 +281,6 @@ class ParserBuffer(bytearray):
     def __init__(self, *args, limit=BUF_LIMIT):
         super().__init__(*args)
 
-        self.offset = 0
-        self.size = 0
         self._limit = limit
         self._exception = None
         self._writer = self._feed_data()
@@ -298,24 +292,12 @@ class ParserBuffer(bytearray):
     def set_exception(self, exc):
         self._exception = exc
 
-    def shrink(self):
-        if self.offset:
-            del self[:self.offset]
-            self.offset = 0
-            self.size = len(self)
-
     def _feed_data(self):
         while True:
             chunk = yield
             if chunk:
-                chunk_len = len(chunk)
-                self.size += chunk_len
                 self.extend(chunk)
 
-            # shrink buffer
-            if (self.offset and len(self) > self._limit):
-                self.shrink()
-
             if self._exception:
                 self._writer = self._feed_data()
                 next(self._writer)
@@ -328,11 +310,10 @@ class ParserBuffer(bytearray):
         """read() reads specified amount of bytes."""
 
         while True:
-            if self.size >= size:
-                start, end = self.offset, self.offset + size
-                self.offset = end
-                self.size = self.size - size
-                return self[start:end]
+            if len(self) >= size:
+                data = self[:size]
+                del self[:size]
+                return bytes(data)
 
             self._writer.send((yield))
 
@@ -340,15 +321,14 @@ class ParserBuffer(bytearray):
         """reads size of less amount of bytes."""
 
         while True:
-            if self.size > 0:
-                if size is None or self.size < size:
-                    size = self.size
+            length = len(self)
+            if length > 0:
+                if size is None or length < size:
+                    size = length
 
-                start, end = self.offset, self.offset + size
-                self.offset = end
-                self.size = self.size - size
-
-                return self[start:end]
+                data = bytes(self[:size])
+                del self[:size]
+                return data
 
             self._writer.send((yield))
 
@@ -359,20 +339,19 @@ class ParserBuffer(bytearray):
         stop_len = len(stop)
 
         while True:
-            pos = self.find(stop, self.offset)
+            pos = self.find(stop)
             if pos >= 0:
                 end = pos + stop_len
-                size = end - self.offset
+                size = end
                 if limit is not None and size > limit:
                     raise errors.LineLimitExceededParserError(
                         'Line is too long.', limit)
 
-                start, self.offset = self.offset, end
-                self.size = self.size - size
-
-                return self[start:end]
+                data = bytes(self[:size])
+                del self[:size]
+                return data
             else:
-                if limit is not None and self.size > limit:
+                if limit is not None and len(self) > limit:
                     raise errors.LineLimitExceededParserError(
                         'Line is too long.', limit)
 
@@ -383,8 +362,8 @@ class ParserBuffer(bytearray):
         then returns data without changing internal buffer."""
 
         while True:
-            if self.size >= size:
-                return self[self.offset:self.offset + size]
+            if len(self) >= size:
+                return bytes(self[:size])
 
             self._writer.send((yield))
 
@@ -396,17 +375,16 @@ class ParserBuffer(bytearray):
         stop_len = len(stop)
 
         while True:
-            pos = self.find(stop, self.offset)
+            pos = self.find(stop)
             if pos >= 0:
-                end = pos + stop_len
-                size = end - self.offset
+                size = pos + stop_len
                 if limit is not None and size > limit:
                     raise errors.LineLimitExceededParserError(
                         'Line is too long. %s' % bytes(self), limit)
 
-                return self[self.offset:end]
+                return bytes(self[:size])
             else:
-                if limit is not None and self.size > limit:
+                if limit is not None and len(self) > limit:
                     raise errors.LineLimitExceededParserError(
                         'Line is too long. %s' % bytes(self), limit)
 
@@ -415,11 +393,10 @@ class ParserBuffer(bytearray):
     def skip(self, size):
         """skip() skips specified amount of bytes."""
 
-        while self.size < size:
+        while len(self) < size:
             self._writer.send((yield))
 
-        self.size -= size
-        self.offset += size
+        del self[:size]
 
     def skipuntil(self, stop):
         """skipuntil() reads until `stop` bytes sequence."""
@@ -429,21 +406,14 @@ class ParserBuffer(bytearray):
         stop_len = len(stop)
 
         while True:
-            stop_line = self.find(stop, self.offset)
+            stop_line = self.find(stop)
             if stop_line >= 0:
-                end = stop_line + stop_len
-                self.size = self.size - (end - self.offset)
-                self.offset = end
+                size = stop_line + stop_len
+                del self[:size]
                 return
-            else:
-                self.size = 0
-                self.offset = len(self) - 1
 
             self._writer.send((yield))
 
-    def __bytes__(self):
-        return bytes(self[self.offset:])
-
 
 class LinesParser:
     """Lines parser.
