@@ -1,3 +1,7 @@
+#!/usr/bin/env python3
+"""Example for aiohttp.web basic server
+"""
+
 import asyncio
 import textwrap
 from aiohttp.web import Application, Response, StreamResponse
